'use strict';

/** @namespace */
/**
 * Statuses of a server Request
 *
 * @typedef {'idle' | 'requested' | 'success' | 'error'} TH_ReqStatus
 */
/**
 * Comm status
 *
 * @typedef {object} TH_CommStatus
 * @property {TH_ReqStatus} status - Status of the request
 * @property {string} text - Text describing current status
 */

/**
 * Message from player with faction selected
 *
 * @typedef {object} TH_MessageFaction
 * @property {import('type/game').TH_Color} color - color of player
 * @property {import('type/game').TH_Faction} faction - The chosen faction
 */
/**
 * Message from player for draft mode steps.
 * Each turn of draft mode, players choses 2 new Ids which will be added to his twelveHeroes.
 *
 * @typedef {object} TH_MessageDraftStep
 * @property {import('type/game').TH_Color} color - color of player
 * @property {number[]} chosenIds - The chosen Ids
 */
/**
 * Message from player for tournament mode.
 * Player chose 12 heroes.
 *
 * @typedef {object} TH_MessageTournamentDeckStep
 * @property {import('type/game').TH_Color} color - color of player
 * @property {number[]} chosenIds - The chosen 12 heroes IDs
 */
/**
 * Message for chat from one player to others
 *
 * @typedef {object} TH_MessageChatFrom
 * @property {string} from - From is the message from
 * @property {string} text - The content of chat message
 */
/**
 * Message from player for ending turn in phase 2 (change up to 3 cards).
 *
 * @typedef {object} TH_MessageReplace3CardsStep
 * @property {import('type/game').TH_Color} color - color of player
 * @property {number[]} heroesInHand - The heroes in hand
 * @property {number[]} heroesInPile - The heroes in pile
 */

exports.unused = {};
